/*

Copyright (C) University of Oxford, 2005-2010

University of Oxford means the Chancellor, Masters and Scholars of the
University of Oxford, having an administrative office at Wellington
Square, Oxford OX1 2JD, UK.

This file is part of Chaste.

Chaste is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published
by the Free Software Foundation, either version 2.1 of the License, or
(at your option) any later version.

Chaste is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
License for more details. The offer of Chaste under the terms of the
License is subject to the License being interpreted in accordance with
English Law and subject to any action against the University of Oxford
being under the jurisdiction of the English Courts.

You should have received a copy of the GNU Lesser General Public License
along with Chaste. If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef _ABSTRACTDATAREADER_HPP_
#define _ABSTRACTDATAREADER_HPP_

#include <string>
#include <vector>

/**
 * An abstract data reader class.
 */
class AbstractDataReader
{
public:

    /**
     * Get the entries for a given variable.
     * This method must be implemented by each subclass.
     *
     * @param rVariableName
     */
    virtual std::vector<double> GetValues(const std::string& rVariableName) = 0;

    /**
     * Get the entries for a given variable with fixed dimension.
     * This method must be implemented by each subclass.
     *
     * @param rVariableName
     * @param fixedDimension
     */
    virtual std::vector<double> GetValues(const std::string& rVariableName, int fixedDimension) = 0;

    /**
     * Base class with virtual methods needs a virtual destructor.
     */
    virtual ~AbstractDataReader()
    {}
};

#endif //_ABSTRACTDATAREADER_HPP_
